# Copyright (c) 2025 Itential, Inc
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
Core package for Itential MCP.

This package contains fundamental modules that provide basic functionality
used throughout the Itential MCP server including logging, metadata,
environment utilities, error handling, and exceptions.
"""
